@@ -38,7 +38,36 @@
   #+clisp `(let ((custom:*suppress-check-redefinition* t)) ,form)
   #-(or sbcl clisp) `(progn ,form))
 
-($put '$wxmaxima `((mlist simp) 0 8 7) '$version)
+($put '$wxmaxima `((mlist simp) 12 4 1) '$version)
+
+(defun $wxbuild_info ()
+  (let ((wxmaxima-version (cdr ($get '$wxmaxima '$version)))
+        (year (sixth cl-user:*maxima-build-time*))
+        (month (fifth cl-user:*maxima-build-time*))
+        (day (fourth cl-user:*maxima-build-time*))
+        (hour (third cl-user:*maxima-build-time*))
+        (minute (second cl-user:*maxima-build-time*))
+        (seconds (first cl-user:*maxima-build-time*)))
+    (format t "wxMaxima version: ~{~d~^.~}~%" wxmaxima-version)
+    (format t "Maxima version: ~a~%" *autoconf-version*)
+    (format t "Maxima build date: ~4,'0d-~2,'0d-~2,'0d ~2,'0d:~2,'0d:~2,'0d~%"
+            year month day hour minute seconds)
+    (format t "Host type: ~a~%" *autoconf-host*)
+    (format t "Lisp implementation type: ~a~%" (lisp-implementation-type))
+    (format t "Lisp implementation version: ~a~%" (lisp-implementation-version)))
+  "")
+
+(defmfun $wxbug_report ()
+  (format t "~%The Maxima bug database is available at~%")
+  (format t "    http://sourceforge.net/tracker/?atid=104933&group_id=4933&func=browse~%")
+  (format t "Submit bug reports by following the 'Add new' link on that page.~%~%")
+  (format t "The wxMaxima bug database is available at~%")
+  (format t "    https://github.com/andrejv/wxmaxima/issues?direction=desc&sort=created&state=open~%")
+  (format t "Submit bug reports by following the 'New issue' link on that page.~%~%")
+  (format t "Please include the following information with your bug report:~%")
+  (format t "-------------------------------------------------------------~%")
+  ($wxbuild_info)
+  (format t "-------------------------------------------------------------~%"))
 
 (setf (get '$inchar 'assign) 'neverset)
 (setf (get '$outchar 'assign) 'neverset)
@@ -65,6 +94,8 @@
          (wxxml-matchfix-dim x l r))
 	((equal (get (caar x) 'dimension) 'dimension-nary)
 	 (wxxml-nary x l r))
+        ((get (caar x) 'defstruct-template)
+         (wxxml-defstruct x l r))
         (t (wxxml-function x l r))))
 
 (defmacro make-tag (val tag)
@@ -187,6 +218,14 @@
               x (cdr x)
               l nil))))
 
+(defun wxxml-defstruct (x l r)
+  (let ((L1 (cdr (get (caar x) 'defstruct-template)))
+        (L2 (cdr x)))
+    (wxxml-function
+     (cons (car x)
+           (mapcar #'(lambda (e1 e2) (if (eq e1 e2) e1 `((mequal) ,e1 ,e2))) L1 L2))
+     l r)))
+
 ;; we could patch this so sin x rather than sin(x), but instead we made
 ;; sin a prefix operator
 (defun wxxml-function (x l r)
@@ -295,6 +334,12 @@
 (defprop mbox wxxml-mbox wxxml)
 (defprop mlabox wxxml-mbox wxxml)
 
+(defprop mbox 10. wxxml-rbp)
+(defprop mbox 10. wxxml-lbp)
+
+(defprop mlabbox 10. wxxml-rbp)
+(defprop mlabbox 10. wxxml-lbp)
+
 (defun wxxml-mbox (x l r)
   (setq l (wxxml (cadr x) (append l '("<hl>")) nil 'mparen 'mparen)
         r (append '("</hl>") r))
@@ -789,7 +834,7 @@
 (defprop mleqp 80. wxxml-rbp)
 
 (defprop mnot wxxml-prefix wxxml)
-(defprop mnot ("<fnm>not</fnm>") wxxmlsym)
+(defprop mnot ("<fnm altCopy=\"not \">not</fnm>") wxxmlsym)
 (defprop mnot "<fnm>not</fnm>" wxxmlword)
 (defprop mnot 70. wxxml-rbp)
 
@@ -815,7 +860,9 @@
 (defprop %derivative 119. wxxml-rbp)
 
 (defun wxxml-derivative (x l r)
-  (if (and $derivabbrev (every #'integerp (odds (cddr x) 0)))
+  (if (and $derivabbrev
+           (every #'integerp (odds (cddr x) 0))
+           (every #'atom (odds (cddr x) 1)))
       (append l (wxxml-d-abbrev x) r)
       (wxxml (wxxml-d x) (append l '("<d>"))
 	     (append '("</d>") r) 'mparen 'mparen)))
@@ -1213,7 +1260,7 @@
 	 (args (cdddr scene))
 	 (images ()))
     (when (integerp a-range)
-      (setq a-range (mfuncall '$makelist 'i 'i 1 a-range)))
+      (setq a-range (cons '(mlist simp) (loop for i from 1 to a-range collect i))))
     (dolist (aval (reverse (cdr a-range)))
       (let ((preamble ($wxplot_preamble))
 	    (system-preamble (get-plot-option-string '$gnuplot_preamble 2))
@@ -1225,7 +1272,7 @@
 	(dolist (arg args)
 	  (if (and (listp arg) (eql (cadr arg) '$gnuplot_preamble))
 	      (setq preamble (format nil "~a; ~a"
-				     preamble (caddr arg)))))
+				     preamble (meval (maxima-substitute aval a (caddr arg)))))))
 	(apply #'$plot2d `(,(meval expr) ,@(mapcar #'meval args)
 			   ((mlist simp) $plot_format $gnuplot)
 			   ((mlist simp) $gnuplot_preamble ,preamble)
@@ -1244,81 +1291,81 @@
 
 (defvar *windows-OS* (string= *autoconf-win32* "true"))
 
-(defun wxanimate-draw (scene)
+(defun get-file-name-opt (scene)
+  (let (opts filename)
+    (loop for opt in scene do
+         (if (and (not (atom opt))
+                  (eq (caar opt) 'mequal)
+                  (eq (cadr opt) '$file_name))
+             (setq filename (caddr opt))
+             (setq opts (cons opt opts))))
+    (values (reverse opts) filename)))
+
+(defun get-pic-size-opt ()
+  (cond
+    ((= ($get '$draw '$version) 1)
+     `(((mequal simp) $pic_width ,($first $wxplot_size))
+       ((mequal simp) $pic_height ,($second $wxplot_size))))
+    (t
+     `(((mequal simp) $dimensions ,$wxplot_size)))))
+
+(defun wxanimate-draw (scenes scene-head)
   (unless ($get '$draw '$version) ($load "draw"))
-  (let* ((scene (cdr scene))
-	 (a (meval (car scene)))
-	 (a-range (meval (cadr scene)))
-	 (args (cddr scene))
-	 (images ()))
-    (when (integerp a-range)
-      (setq a-range (mfuncall '$makelist 'i 'i 1 a-range)))
-    (dolist (aval (reverse (cdr a-range)))
-      (let* ((filename (wxplot-filename nil))
-	     (*windows-OS* t)
-	     (args (cons '($gr2d)
-                         (mapcar #'(lambda (arg) (meval (maxima-substitute aval a arg)))
-                                 args))))
-	(setq images (cons (format nil "~a.png" filename) images))
-	($apply '$draw
-                (append
-                 `((mlist simp)
-                   ((mequal simp) $terminal $png)
-                   ((mequal simp) $file_name ,filename))
-                 (cond
-                   ((= ($get '$draw '$version) 1)
-                    `(((mequal simp) $pic_width ,($first $wxplot_size))
-                      ((mequal simp) $pic_height ,($second $wxplot_size))))
-                   (t
-                    `(((mequal simp) $dimensions ,$wxplot_size))))
-                 (list args)))))
-    (when images
-      ($ldisp (list '(wxxmltag simp) (format nil "~{~a;~}" images) "slide"))))
-  "")
+  (multiple-value-bind (scene file-name) (get-file-name-opt (cdr scenes))
+    (let* ((a (meval (car scene)))
+           (a-range (meval (cadr scene)))
+           (*windows-OS* t)
+           (args (cddr scene))
+           (images ()))
+      (when (integerp a-range)
+        (setq a-range (cons '(mlist simp) (loop for i from 1 to a-range collect i))))
+      (if file-name
+          ;; If file_name is set, draw the animation into gif using gnuplot
+          (let (imgs)
+            (dolist (aval (reverse (cdr a-range)))
+              (setq imgs (cons
+                          (cons scene-head
+                                (mapcar #'(lambda (arg) (meval (maxima-substitute aval a arg)))
+                                        args))
+                          imgs)))
+            ($apply '$draw
+                    (append
+                     `((mlist simp)
+                       ((mequal simp) $terminal $animated_gif)
+                       ((mequal simp) $file_name ,file-name))
+                     (get-pic-size-opt)
+                     imgs))
+            "")
+          ;; If file_name is not set, show the animation in wxMaxima
+          (progn
+            (dolist (aval (reverse (cdr a-range)))
+              (let* ((filename (wxplot-filename nil))
+                     (args (cons scene-head
+                                 (mapcar #'(lambda (arg) (meval (maxima-substitute aval a arg)))
+                                         args))))
+                (setq images (cons (format nil "~a.png" filename) images))
+                ($apply '$draw
+                        (append
+                         `((mlist simp)
+                           ((mequal simp) $terminal $png)
+                           ((mequal simp) $file_name ,filename))
+                         (get-pic-size-opt)
+                         (list args)))))
+            (when images
+              ($ldisp (list '(wxxmltag simp) (format nil "~{~a;~}" images) "slide")))))
+      "")))
 
 (defmspec $wxanimate_draw (scene)
-  (wxanimate-draw scene))
+  (wxanimate-draw scene '($gr2d)))
 
 (defmspec $with_slider_draw (scene)
-  (wxanimate-draw scene))
-
-(defun wxanimate-draw3d (scene)
-  (unless ($get '$draw '$version) ($load "draw"))
-  (let* ((scene (cdr scene))
-	 (a (meval (car scene)))
-	 (a-range (meval (cadr scene)))
-	 (args (cddr scene))
-	 (images ()))
-    (when (integerp a-range)
-      (setq a-range (mfuncall '$makelist 'i 'i 1 a-range)))
-    (dolist (aval (reverse (cdr a-range)))
-      (let* ((filename (wxplot-filename nil))
-	     (*windows-OS* t)
-	     (args (cons '($gr3d)
-			  (mapcar #'(lambda (arg) (meval (maxima-substitute aval a arg)))
-				  args))))
-	(setq images (cons (format nil "~a.png" filename) images))
-	($apply '$draw
-                (append
-                 `((mlist simp)
-                   ((mequal simp) $terminal $png)
-                   ((mequal simp) $file_name ,filename))
-                 (cond
-                   ((= ($get '$draw '$version) 1)
-                    `(((mequal simp) $pic_width ,($first $wxplot_size))
-                      ((mequal simp) $pic_height ,($second $wxplot_size))))
-                   (t
-                    `(((mequal simp) $dimensions ,$wxplot_size))))
-                 (list args)))))
-    (when images
-      ($ldisp (list '(wxxmltag simp) (format nil "~{~a;~}" images) "slide"))))
-  "")
+  (wxanimate-draw scene '($gr2d)))
 
 (defmspec $with_slider_draw3d (scene)
-  (wxanimate-draw3d scene))
+  (wxanimate-draw scene '($gr3d)))
 
 (defmspec $wxanimate_draw3d (scene)
-  (wxanimate-draw3d scene))
+  (wxanimate-draw scene '($gr3d)))
 
 (defun $wxplot2d (&rest args)
   (let ((preamble ($wxplot_preamble))
@@ -1449,15 +1496,27 @@
 (defun $wxxmlput (e s &optional tx lbp rbp)
   (cond (($listp s)
 	 (setq s (margs s)))
-	(t
-	 (setq s (list ($sconcat s)))))
-  (setq s (mapcar #'wxxml-stripdollar s))
+	((atom s)
+	 (setq s (list (wxxml-stripdollar ($sconcat s))))))
   (cond ((or (null lbp) (not (integerp lbp)))
          (setq lbp 180)))
   (cond ((or (null rbp) (not (integerp rbp)))
          (setq rbp 180)))
   (cond ((null tx)
-	 (putprop e (nth 0 s) 'wxxmlword))
+         (if (stringp (nth 0 s))
+             (putprop e (nth 0 s) 'wxxmlword)
+             (let ((fun-name (gensym))
+                   (fun-body
+                    `(append l
+                             (list
+                              (let ((f-x (mfuncall ',s x)))
+                                (if (stringp f-x)
+                                    f-x
+                                    (merror "wxxml: function ~s did not return a string.~%"
+                                            ($sconcat ',(nth 0 s))))))
+                             r)))
+               (setf (symbol-function fun-name) (coerce `(lambda (x l r) ,fun-body) 'function))
+               (setf (get e 'wxxml) fun-name))))
 	((eq tx '$matchfix)
 	 (putprop e 'wxxml-matchfix 'wxxml)
 	 (cond ((< (length s) 2)
@@ -1547,7 +1606,8 @@
 	(args (mapcar (lambda (u)
 			(cond ((atom u) (symbol-to-string u))
 			      ((eq (caar u) 'mlist)
-			       ($concat "[" (symbol-to-string (cadr u)) "]"))
+			       ($concat "[" (symbol-to-string
+                                             (if (atom (cadr u)) (cadr u) (cadadr u))) "]"))
 			      (t (symbol-to-string (cadr u)))))
 		      (cdr fun))))
     (format nil "FUNCTION: ~a$TEMPLATE: ~a(~{<~a>~^, ~})" fun-name fun-name args)))
@@ -1630,12 +1690,18 @@
       (setq mat (cons '(matrix simp)
 		      (cons col-names (cdr mat))))
       (setq mtrx (cons 'colnames mtrx)))
-    (cons (append '($matrix simp) mtrx) (cdr mat))))
+    ($ldisp (cons (append '($matrix simp) mtrx) (cdr mat)))
+    '$done))
 
 ;; Load the initial functions (from mac-init.mac)
 (let ((*print-circle* nil))
   (format t "<wxxml-symbols>~{~a~^$~}</wxxml-symbols>"
 	  (mapcar #'$print_function (cdr ($append $functions $macros)))))
 
+(no-warning
+ (defun mredef-check (fnname)
+   (declare (ignore fnname))
+   t))
+
 (when ($file_search "wxmaxima-init")
   ($load "wxmaxima-init"))
